---
html:
  toc: true
  offline: true
export_on_save:
  html: true
---
# KSB 웹툴킷 설치
---

UI 기반으로 KSB 인공지능 프레임워크를 사용할 수 있는 KSB 웹툴킷 설치 과정을 설명합니다.

## 설치 요구사항

- Docker 기반 KSB 툴박스 환경에서는, KSB 웹툴킷 실행을 위한 기능들이 사전에 설치 되어 있거나 설치 과정에 포함 되어 있으므로, 본 과정이 필요하지 않습니다.
- Ubuntu 16.04가 설치된 메모리 16GB 이상 데스트탑이나 노트북이 요구됩니다.
- KSB 웹툴킷은 KSB 인공지능 프레임워크를 기반으로 구동됩니다. 따라서 startKsbApiServing 을 실행하여 KSB 인공지능 프레임워크를 구동하고 있어야 합니다.
- KSB 웹툴킷은 저장소로 HDFS를 사용합니다. 따라서, 저장소로 사용할 PC에 Hadoop 및 관련 소프트웨어가 설치 되어 있어야 합니다. 설치 방법은 다음 링크를 참고 합니다. ([Hadoop 2.7.3 설치하기](https://csleoss.etri.re.kr/images/contents/manual_1.0/1.3.HowToInstallKsbFrameWork.html#host-pc-hadoop-273-%EC%84%A4%EC%B9%98%ED%95%98%EA%B8%B0))
Hadoop을 설치한 후에는 ksb-csle/conf/ksb.conf 파일에서 hadoop.master 정보를 설치한 호스트명으로 설정해 주어야 합니다.
또한 hadoop.hdfs.activated 도 true로 설정해야 정상적으로 HDFS 를 저장소로 이용할 수 있습니다.(기본적으로 true로 설정 되어 있습니다.)
- DB에서 한글을 정상적으로 사용하기 위해, 호스트 운영체제의 언어를 한글로 설정 합니다. 변경 하는 방법은 다음 링크를 참고 합니다. ([Ubuntu 언어 변경](https://askubuntu.com/questions/509174/how-can-i-change-the-language-from-english-to-spanish))
- Postgresql 9.6.8 이상이 설치되어야 합니다.
- Pgadmin3 1.22.2 이상이 설치되어야 합니다.
- JDK 8 이 설치되어야 합니다. 설치 방법은 다음 링크를 참고 합니다. ([JDK 8 설치](https://csleoss.etri.re.kr/images/contents/manual_1.0/1.2.HowToInstallKsbToolBox.html#host-pc-java-8-oracle-%EC%84%A4%EC%B9%98%ED%95%98%EA%B8%B0))
- Apache Tomcat 7.x가 설치되어야 합니다.

## KSB 인공지능 프레임워크 DBMS (Postgresql) 설치하기

KSB 인공지능 프레임워크는 프레임워크 사용자 정보, 생성 및 구동된 워크플로우 정보, KSB 웹툴킷에서 이용하는 UI Component 관련 정보를 관리하기 위해 Postgresql을 사용 합니다.
Postgresql 설치 버전은 9.6.8 이상를 권장하며, 다음과 같이 설치한 후 재시작합니다.

```sh
$ sudo apt-get update

$ sudo apt-get install -y software-properties-common

$ sudo add-apt-repository "deb http://apt.postgresql.org/pub/repos/apt/ xenial-pgdg main"

$ wget --quiet -O - https://www.postgresql.org/media/keys/ACCC4CF8.asc | sudo apt-key add -

$ sudo apt-get update

$ sudo apt-get install -y postgresql-9.6

$ sudo service postgresql restart
```

또한 Postgresql DB 접근 및 이용을 편리하게 하기 위해 Pgadmin을 설치하여 사용합니다. pgadmin3는 1.22.2 이상을 설치합니다. (Postgresql 9.6.8 이상을 설치해야 잘 동작합니다.) 툴박스 내에서 제공하는 DB restore 파일 등도 위 두 버전을 통해 백업 되었습니다.

```sh
$ sudo apt-get update

$ sudo apt-get install -y pgadmin3
```

## 외부에서 DB 접근 가능하도록 수정하기

설치된 Postgresql은 기본적으로는 localhost에서만 사용 가능합니다. 외부에서도 접근 가능하도록 하기 위해서는 다음 두 파일을 수정해야 합니다.

```sh
** postgresql remote access

$ sudo vi /etc/postgresql/9.6/main/pg_hba.conf/pg_hba.conf

#host    all             all             127.0.0.1/32            md5
host    all             all             0.0.0.0/0            md5

$ sudo vi /etc/postgresql/9.6/main/postgresql.conf

#listen_addresses = 'localhost'
listen_addresses = '*'

$ sudo service postgresql restart
```

## KSB 인공지능 프레임워크 DB 정의

Postgresql 설치가 완료되면, DB에 접속하여 DB 사용자 계정과 권한을 생성하고, 백업파일을 이용하여 KSB 인공지능 프레임워크 DB 정보를 restore 합니다.

#### KSB 인공지능 프레임워크 DB 생성
```sh
# create user csle
$ sudo -u postgres bash -c "psql -c \"CREATE USER csle password '사용자암호' superuser;\""

# alter csle as a superuser
$ sudo -u postgres bash -c "psql -c \"ALTER USER csle WITH CREATEROLE REPLICATION CREATEDB SUPERUSER;\""

# create database csledb
$ sudo -u postgres bash -c "psql -c \"CREATE DATABASE csledb WITH OWNER = csle ENCODING = 'UTF-8' CONNECTION LIMIT = -1;\""
```

#### Restore 방법 1: Postgresql 9.6.8 에서 백업 pg_dump를 통해 백업된 파일을 pg_restore로 restore 하기
백업 파일: [csledb_backup_20190110_1_19.03.backup](https://csleoss.etri.re.kr/images/contents/manual_1903/csledb_backup_20190110_1_19.03.backup)

- 본 파일은 pgadmin3 상에서도 restore 할 수 있습니다.

```sh
$ PGPASSWORD=사용자암호 pg_restore -h localhost -U csle -C -d csledb /home/csle/Downloads/csledb_backup_20190110_1_19.03.backup
```


#### Restore 방법 2: PSQL로 백업된 sql로 restore 하기
백업 파일: [csledb_backup_plain_20190110_1_19.03.backup](https://csleoss.etri.re.kr/images/contents/manual_1903/csledb_backup_plain_20190110_1_19.03.backup)

- SQL로 Restore 하는 방법은, pg_dump, pg_restore와는 달리 버전에 종속적이지 않습니다.

```sh
$ sudo -su postgres

$ psql

# backup 파일 내에 csledb를 생성하고, csledb에 connect 하는 명령이 포함되어 있음. 따라서, 'create database csledb' 과정을 생략해도 됨
$ \i /home/csle/Downloads/csledb_backup_plain_20190110_1_19.03.backup
```

postgresql 에 접속하여 정상적으로 복구 되었는지 확인 합니다.

```sh
$ sudo -su postgres

$ psql

postgres=# \c csledb;

csledb=# csledb=# select * from "VersionMap" where "KsbVersion" = '19.03';

 MsgVersion | KsbVersion | IsCurrent | Description  | Default
------------+------------+-----------+--------------+---------
 1.0        | 19.03      |         1 | branch 19.03 |       1
(1개 행)

```

## KSB 웹툴킷 DB 정의

KSB 웹툴킷 DB를 생성하고, 백업파일을 이용하여 KSB 웹툴킷 DB 정보를 restore 합니다.

#### KSB 웹툴킷 DB 생성
```sh
$ sudo -u postgres bash -c "psql -c \"CREATE DATABASE uidb WITH OWNER = csle ENCODING = 'UTF-8' CONNECTION LIMIT = -1;\""

```

#### Restore 방법 1: Postgresql 9.6.8 에서 백업 pg_dump를 통해 백업 된 파일을 pg_restore로 restore 하기
백업 파일: [uidb_backup_20190110_19.03.backup](https://csleoss.etri.re.kr/images/contents/manual_1903/uidb_backup_20190110_19.03.backup)
- 본 파일은 pgadmin3 상에서도 restore 할 수 있습니다.

```sh
$ PGPASSWORD=사용자암호 pg_restore -h localhost -U csle -C -d uidb /home/csle/Downloads/uidb_backup_20190110_19.03.backup
```


#### Restore 방법 2: PSQL로 백업 된 sql로 restore 하기
백업 파일: [uidb_backup_plain_20190110_19.03.backup](https://csleoss.etri.re.kr/images/contents/manual_1903/uidb_backup_plain_20190110_19.03.backup)
- SQL로 Restore 하는 방법은, pg_dump, pg_restore와는 달리 버전에 종속적이지 않습니다.

```sh
$ sudo -su postgres

$ psql

# backup 파일 내에 uidb를생성하고, uidb에 connect 하는 명령이 포함되어 있음. 따라서, 'KSB 웹툴킷 DB 생성' 과정을 생략해도 됨
$ \i /home/csle/Downloads/uidb_backup_plain_20190110_19.03.backup
```


postgresql 에 접속하여 정상적으로 복구 되었는지 확인 합니다.

```sh
$ sudo -su postgres

$ psql

postgres=# \c uidb;

uidb=# SELECT count(*) FROM template where msg_version = '1.0';

 count
-------
    23
(1개 행)

```

## Apache Tomcat 다운로드

KSB 웹툴킷은 웹컨테이너로 Apache tomcat 7을 이용합니다. https://tomcat.apache.org/download-70.cgi 사이트에서 binary distribution을 다운로드 받아, 적당한 위치에 압축을 해제 합니다.

본 매뉴얼 에서는 '/home/csle/ui_of_csle/apache-tomcat-7.0.88/' 에
압축을 해제 한 것으로 가정합니다.

## Tomcat 설정 업데이트

### web.xml 파일 업데이트
/home/csle/ui_of_csle/apache-tomcat-7.0.88/conf/ 디렉토리에 있는 web.xml 파일을 열어 UTF-8 설정 및 Session timeout을 조정합니다.

```xml
<filter>
    <filter-name>setCharacterEncodingFilter</filter-name>
    <filter-class>org.apache.catalina.filters.SetCharacterEncodingFilter</filter-class>
    <init-param>
        <param-name>encoding</param-name>
        <param-value>UTF-8</param-value>
    </init-param>
    <async-supported>true</async-supported>
</filter>

.....

<filter-mapping>
    <filter-name>setCharacterEncodingFilter</filter-name>
    <url-pattern>/*</url-pattern>
</filter-mapping>

.....

<session-config>
    <session-timeout>360</session-timeout> <!-- minutes -->
</session-config>
```

### server.xml 파일 업데이트
다음으로, server.xml 파일을 열어 UTF-8 설정을 조정 합니다.

```xml
<Connector port="8080" protocol="HTTP/1.1"
           connectionTimeout="2000000"
           redirectPort="8443" disableUploadTimeout="true" URIEncoding="UTF-8"/>

.....

<Connector port="8009" protocol="AJP/1.3" redirectPort="8443" URIEncoding="UTF-8"/>
```



## 웹파일 설치 및 KSB 인공지능 프레임워크 DB 접속정보 설정

웹파일 [csle-web_20190304.war](https://csleoss.etri.re.kr/images/contents/manual_1903/csle-web_20190304.war) 을 다운 받아 /home/csle/ui_of_csle/apache-tomcat-7.0.88/webapps 디렉토리로 복사 합니다.
그리고, csle-web_20190304.war 파일의 이름을 ROOT.war로 변경 합니다.

### Tomcat 서버 구동
/home/csle/ui_of_csle/apache-tomcat-7.0.88/bin 디렉토리에 있는
``catalina.sh`` 파일을 통해 Tomcat을 구동하면 웹파일이 압축해제 되어
/home/csle/ui_of_csle/apache-tomcat-7.0.88/webapps/ROOT 폴더가 생성되고, 서버가 구동 됩니다.

```sh
$ cd /home/csle/ui_of_csle/apache-tomcat-7.0.88/bin
$ ./catalina.sh start -Dfile.encoding=UTF-8

(2018-06-30 12:22:37) <localhost-startStop-1> [DEBUG] SQL: SELECT * FROM system_configuration WHERE name = ?, name: [csle.gw.host] (SystemConfigurationDao.java:130)
(2018-06-30 12:22:37) <localhost-startStop-1> [DEBUG] SQL: SELECT * FROM system_configuration WHERE name = ?, name: [csle.gw.port] (SystemConfigurationDao.java:130)
6월 30, 2018 12:22:38 오후 org.apache.catalina.startup.HostConfig deployWAR
정보: Deployment of web application archive /home/csle/ui_of_csle/apache-tomcat-7.0.81/webapps/ROOT.war has finished in 5,246 ms
6월 30, 2018 12:22:38 오후 org.apache.coyote.AbstractProtocol start
정보: Starting ProtocolHandler ["http-bio-8080"]
6월 30, 2018 12:22:38 오후 org.apache.coyote.AbstractProtocol start
정보: Starting ProtocolHandler ["ajp-bio-8009"]
6월 30, 2018 12:22:38 오후 org.apache.catalina.startup.Catalina start
정보: Server startup in 5339 ms
```

### Tomcat 로그 보기
Tomcat 이 잘 구동되고 있는지 확인하기 위해 Tomcat 로그 파일을 엽니다.

```sh
$ tail -f ../logs/catalina.out
```

## 구글 Chrome 브라우저 설치하기
KSB 웹툴킷은 기본적으로 HTML5 호환 브라우저에서 동작하도록 개발 되었으나, 일부 기능은 Chrome 브라우저에서만 이용 가능합니다. 따라서, Chrome 브라우저를 이용하기를 권장 합니다. 다음 절차를 통해 Chrome 브라우저 최신 버전을 설치 합니다.

```sh
# 운영체제와 CPU 아키텍처에 따라 별도의 링크를 통해 다운 받아야 할 수도 있습니다.
$ wget https://dl.google.com/linux/direct/google-chrome-stable_current_amd64.deb

$ sudo apt-get install -y libxss1 libgconf2-4 libappindicator1 libindicator7

$ sudo dpkg -i google-chrome-stable_current_amd64.deb
```

## KSB 인공지능 프레임워크 구동

KSB 인공지능 프레임워크 툴박스를 설치하거나 소스를 빌드한 후,
ksb-csle/bin 디렉토리에 있는 startKsbApiServing.sh 파일을 실행 합니다.
파일 실행 전에 ls와 같이 간단한 명령을 sudo를 통해 한 번 실행 합니다.

```sh
$ sudo ls
$ ./startKsbApiServing.sh
```

구동이 성공적으로 이루어지면, 다음과 같은 로그가 생성되며,
[akka://KsbApi-serving/user/IO-HTTP/listener-0] Bound to /0.0.0.0:19999
위 메시지를 통해 현재 호스트 PC의 19999 포트를 통해 구동 되는 것을 확인할 수 있습니다.

```sh
2018-10-22 16:51:12.113 [INFO] [ConfigUtils$] KSB config file path :/home/csle/ksb-csle/conf/ksb.conf (ConfigUtils.scala:29)
2018-10-22 16:51:13.355 [INFO] [KsbApiServing$] [Local mode] Update status of incompletely terminated [0] job(s) in previous operation (KsbApiServing.scala:60)
2018-10-22 16:51:13.644 [INFO] [ApiServingActor] KSB ApiServingActor has been started. (ApiServingActor.scala:820)
2018-10-22 16:51:13.644 [INFO] [ApiServingActor] KSB ApiServingActor has been started. (ApiServingActor.scala:820)
2018-10-22 16:51:13.644 [INFO] [ApiServingActor] KSB ApiServingActor has been started. (ApiServingActor.scala:820)
2018-10-22 16:51:13.644 [INFO] [ApiServingActor] KSB ApiServingActor has been started. (ApiServingActor.scala:820)
2018-10-22 16:51:13.644 [INFO] [ApiServingActor] KSB ApiServingActor has been started. (ApiServingActor.scala:820)
2018-10-22 16:51:13,905 INFO StdSchedulerFactory: Using default implementation for ThreadExecutor
2018-10-22 16:51:13,910 INFO SimpleThreadPool: Job execution threads will use class loader of thread: main
2018-10-22 16:51:13,923 INFO SchedulerSignalerImpl: Initialized Scheduler Signaller of type: class org.quartz.core.SchedulerSignalerImpl
2018-10-22 16:51:13,923 INFO QuartzScheduler: Quartz Scheduler v.2.3.0 created.
2018-10-22 16:51:13,924 INFO RAMJobStore: RAMJobStore initialized.
2018-10-22 16:51:13,925 INFO QuartzScheduler: Scheduler meta-data: Quartz Scheduler (v2.3.0) 'DefaultQuartzScheduler' with instanceId 'NON_CLUSTERED'
  Scheduler class: 'org.quartz.core.QuartzScheduler' - running locally.
  NOT STARTED.
  Currently in standby mode.
  Number of jobs executed: 0
  Using thread pool 'org.quartz.simpl.SimpleThreadPool' - with 10 threads.
  Using job-store 'org.quartz.simpl.RAMJobStore' - which does not support persistence. and is not clustered.

2018-10-22 16:51:13,925 INFO StdSchedulerFactory: Quartz scheduler 'DefaultQuartzScheduler' initialized from default resource file in Quartz package: 'quartz.properties'
2018-10-22 16:51:13,925 INFO StdSchedulerFactory: Quartz scheduler version: 2.3.0
2018-10-22 16:51:13,925 INFO QuartzScheduler: Scheduler DefaultQuartzScheduler_$_NON_CLUSTERED started.
[INFO] [10/22/2018 16:51:13.979] [KsbApi-serving-akka.actor.default-dispatcher-2] [akka://KsbApi-serving/user/IO-HTTP/listener-0] Bound to /0.0.0.0:19999
[INFO] [10/22/2018 16:51:13.981] [KsbApi-serving-akka.actor.default-dispatcher-3] [akka://KsbApi-serving/deadLetters] Message [akka.io.Tcp$Bound] from Actor[akka://KsbApi-serving/user/IO-HTTP/listener-0#1547261392] to Actor[akka://KsbApi-serving/deadLetters] was not delivered. [1] dead letters encountered. This logging can be turned off or adjusted with configuration settings 'akka.log-dead-letters' and 'akka.log-dead-letters-during-shutdown'.
2018-10-23 08:22:18,351 WARN NativeCodeLoader: Unable to load native-hadoop library for your platform... using builtin-java classes where applicable
```

## KSB 웹툴킷 설정 업데이트

KSB 웹툴킷과 KSB 인공지능 프레임워크는 다음 사용자 정보가 저장되어 있습니다.

&nbsp;&nbsp;&nbsp;&nbsp;User ID: ksbuser@etri.re.kr / User Password: ksbuser@etri.re.kr

Chrome 브라우저(또는 HTML5 표준 호환 브라우저)를 열고 위 사용자 정보를 입력하여 로그인합니다.

```sh
http://localhost:8080
```
![login](./images/1.4_01.png)

정상적으로 로그인에 성공하면, Management 메뉴를 클릭해서 KSB 인공지능 프레임워크가 구동되고 있는 환경 정보를 csle.gw.host 와 csle.gw.port 정보에 반영 합니다.
본 매뉴얼에서는 localhost 와 19999 포트가 되겠습니다.

![management](./images/1.4_02.png)

설정이 완료되면, Tomcat 서버를 재구동 하고, KSB 웹툴킷을 이용 합니다.
```sh

$ ./catalina.sh stop

$ ./catalina.sh start

```

## 구글 Chrome 브라우저에서 캐시 제거

KSB 웹툴킷을 업데이트 하면, Chrome에서 처음에는 정상적으로 동작하지 않을 수 있습니다. Chrome이 이전 정보를 캐싱 하고 있기 때문인데요. 그럴때는 다음 절차로 캐시를 지워 주시기 바랍니다.

- KSB 웹툴킷 사이트를 오픈한 상태에서 F12 버튼을 누른다
- Chrome 의 새로고침 버튼 위에서 마우스 오른쪽 버튼을 누른 다음, '캐시 비우기 및 강력 새로고침' 을 선택한다.


## KSB 웹툴킷 회원 가입 기능 활성화를 위한 설정

KSB 웹툴킷은 이메일을 통해 사용자 가입을 처리하도록 되어 있습니다. 이를 위해 관리자의 메일 계정을 설정 하도록 되어 있으며, Gmail 계정을 통해 설정할 수 있습니다. 관리자는 uidb 의 system_configuration 테이블에 자신의 Gmail 계정을 설정할 수 있으며, 사용자가 회원 가입을 신청하면 관련 메일이 관리자의 Gmail 계정을 통해 발송 됩니다.
설정 방법은, KSB 웹툴킷에 관리자 계정(ksbuser@etri.re.kr)으로 로그인 하여 Management 메뉴의 System Configuration 탭에서 'javax.mail.session.prop.id' property에서 수정아이콘을 클릭하여 csle.manager 을 'javax.mail.session.prop.password' 에 csle1234를 입력하고 저장 아이콘을 클릭하여 수정한 사항을 저장 합니다.

이메일의 발송 시, 사용자에게 메일로 안내 되는 링크의 ip는 KSB 웹툴킷이 구동 중인 서버의 설정을 따릅니다. 만약 localhost로 구동 하였다면, KSB 웹툴킷이 구동 중인 서버에서만 가입 절차를 진행할 수 있으므로, 외부에서 접속 가능한 ip로 실행 하시기 바랍니다.
