---
html:
  toc: true
  offline: true
export_on_save:
  html: true
---
# 호스트 PC의 SSH 포트와 hostname을 사용자가 원하는 설정으로 변경하기 위한 KSB 웹툴킷 설정 방법
---

현재 배포한 KSB 웹툴킷은 호스트 PC의 SSH 포트를 2243으로 변경해서 사용합니다.
왜냐하면 KSB 웹툴킷의 도커 컨테이너에서 SSH 22번을 사용하기 때문에 포트 충돌을 피하기 위함입니다.

만약 사용자가 호스트 PC의 SSH 포트를 22번을 유지하고 KSB 웹툴킷의 SSH 포트를 2243번으로 변경하여 사용하기를 원하는 경우를 위해 아래의 메뉴얼을 제공합니다.
또한 호스트 PC의 hostname을 csle1으로 사용하지 않고 사용자가 원하는 이름으로 사용하는 설정 방법도 함께 설명합니다.

만약 자신의 PC 환경이 아래와 같다고 가정한 경우, HOST PC에서 수정해야 하는 사항과 KSB 웹툴킷 docker 이미지를 변경하는 방법에 대해서 알려드립니다.

[중요] hostname은 소문자로 설정하시기 바랍니다. hadoop을 사용한다면 uppercase 사용시 문제가 됩니다.
```sh
HOST PC hostname  : csleserver
HOST PC IP        : 192.168.1.102
HOST PC SSH PORT  : 22
HOST PC user name : csle
```

## [Host PC] csle 사용자 계정(권한: administrator) 생성하기
```sh
sudo adduser csle
    Adding user 'csle' ...
    Adding new group 'csle' (1001) ...
    Adding new user 'csle' (1001) with group 'csle' ...
    Creating home directory '/home/csle' ...
    Copying files from '/etc/skel' ...
    새 UNIX 암호 입력:
    새 UNIX 암호 재입력:
    passwd: password updated successfully
    Changing the user information for csle
    Enter the new value, or press ENTER for the default
    Full Name []:
    Room Number []:
    Work Phone []:
    Home Phone []:
    Other []:
    Is the information correct? [Y/n] Y
```
/etc/sudoers를 열어 csle 계정에 administrator 권한을 추가합니다.
"경고: 읽기 전용 파일을 고치고 있습니다" 경고는 무시하고 저장하고 나옵니다.
```sh
sudo vi /etc/sudoers
    # User privilege specification
    root    ALL=(ALL:ALL) ALL
    # 맨 아래줄에 내용을 추가합니다.
    #includedir /etc/sudoers.d
    csle    ALL=(ALL) NOPASSWD: ALL
```

## [Host PC] openjdk-8 설치하기
Host PC에 Ubuntu 16.04.5를 처음 설치한 경우 java openjdk 8을 설치합니다. 아래의 내용을 터미널에 복사하여 설정 가능합니다.
```sh
sudo apt-get update && \
sudo apt-get -y install openjdk-8-jre && \
sudo apt-get -y install openjdk-8-jdk
```

터미널을 다시 시작한 후, 아래의 명령으로 java 설치 여부를 확인합니다.

```sh
java -version
java version "1.8.0_212"
Java(TM) SE Runtime Environment (build 1.8.0_212-b13)
Java HotSpot(TM) 64-Bit Server VM (build 25.181-b13, mixed mode)
```

만약 apt-get update시 에러가 발생하면, repo 주소를 변경합니다.
sudo vi /etc/apt/sources.list
```sh
:%s/kr.archive.ubuntu.com/mirror.kakao.com/
:%s/security.ubuntu.com/mirror.kakao.com/
```

## [Host PC] KSB 툴박스 관련 프로그램 설치하기
Host PC에 아래의 명령으로 KSB 관련 프로그램을 설치합니다. 아래의 내용을 터미널에 복사하여 설정가능합니다.

```sh
sudo apt-get install -y --no-install-recommends apt-utils curl bc jq && \
sudo apt-get install -y openssh-server locales wget git vim rsync locales \
filezilla python3-pip && \
sudo apt-get install -y net-tools && \
pip3 install kafka-python pillow requests numpy
```

## [Host PC] SSH port 변경 및 root 로그인 가능하도록 SSHD config 수정하기
sudo vi /etc/ssh/sshd_config 명령을 이용해서 자신이 원하는 port로 수정합니다.

```sh
Port 22

LoginGraceTime 120
PermitRootLogin yes
StrictModes yes
```

## [Host PC] SSH config 수정하기 (known_hosts에 호스트 저장 질문을 하지 않도록 설정)
아래의 명령을 수행하여 ssh_config의 기존 내용을 모두 삭제하고 설정을 추가합니다. 아래의 내용을 터미널에 복사하여 설정가능합니다.

```sh
sudo truncate -s0 /etc/ssh/ssh_config
echo "Host localhost" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "StrictHostKeyChecking no" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "Host 0.0.0.0" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "StrictHostKeyChecking no" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "Host 127.0.0.1" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "StrictHostKeyChecking no" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "Host csle*" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "StrictHostKeyChecking no" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "UserKnownHostsFile=/dev/null" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "Host master" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "StrictHostKeyChecking no" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
echo "UserKnownHostsFile=/dev/null" \
| sudo tee -a /etc/ssh/ssh_config >/dev/null
sudo service ssh restart  
```

## [Host PC] Docker 설치하기

Host PC에 docker-ce 프로그램을 설치하기 위해 아래 명령어를 순차적으로 입력합니다. 아래의 내용을 터미널에 복사하여 설정가능합니다.

```sh
curl -fsSL https://download.docker.com/linux/ubuntu/gpg | \
sudo apt-key add - && \
sudo add-apt-repository \
"deb [arch=amd64]
https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable" && \
sudo apt-get update && \
apt-cache policy docker-ce && \
sudo apt-get install -y docker-ce=18.06.1~ce~3-0~ubuntu && \
sudo systemctl status docker
```

#### [참고사항] 기존 docker-engine 삭제
```sh
sudo apt-get purge -y docker docker-engine docker.io
```

Docker 명령시 root 권한이 필요합니다. root가 아닌 사용자가 sudo 없이 사용하려면 해당 사용자를 docker 그룹에 추가합니다.

```sh
sudo usermod -aG docker csle
# 사용자가 로그인 중이라면 다시 로그인 후 권한이 적용됩니다.
# 단, 현재 터미널에서만 적용됩니다.
sudo su csle
PC 재부팅
```
PC 재부팅을 통해서 sudo 명령없이 docker 명령을 사용할 수 있습니다.


## [Host PC] /etc/hosts 수정하기
아래와 같이 자신의 hostname을 추가 설정합니다.
```sh
127.0.0.1   localhost
# 아래 주석처리. Hdfs 연동시 문제가 생김.
#127.0.1.1  csleserver

# The following lines are desirable for IPv6 capable hosts
::1     ip6-localhost ip6-loopback
fe00::0 ip6-localnet
ff00::0 ip6-mcastprefix
ff02::1 ip6-allnodes
ff02::2 ip6-allrouters

# 사용자는 변경하고자 하는 Host PC IP 주소로 수정
192.168.1.102 csleserver master
```

## [Host PC] bashrc 설정 추가하기
csle 계정의 bashrc 파일을 업데이트 합니다. 아래의 내용을 터미널에 복사하여 설정가능합니다.

```sh
cat <<EOT >> ~/.bashrc
export JAVA_HOME=/usr/lib/jvm/java-8-openjdk-amd64
export PATH=\$PATH:\$JAVA_HOME/bin
export KSB_HOME=/home/csle/ksb-csle
export PYTHONPATH=/home/csle/ksb-csle/pyML/:\$PYTHONPATH
export PYTHONPATH=./:/home/csle/ksb-csle/ksblib:\$PYTHONPATH
EOT
source ~/.bashrc
```


## [Host PC] 크롬 설치하기 (optional)
KSB 툴박스의 웹툴킷을 안정적으로 사용하기 위해서 크롬 브라우저를 설치합니다. 아래의 내용을 터미널에 복사하여 설정가능합니다.

```sh
cd && \
wget \
https://dl.google.com/linux/direct/google-chrome-stable_current_amd64.deb
sudo apt-get update && \
sudo apt-get install -y libxss1 libappindicator1 libindicator7 && \
sudo dpkg -i google-chrome-stable_current_amd64.deb
```

## [Host PC] KSB 툴박스 설치 및 docker image 다운받기
<a href="http://csleoss.etri.re.kr:8088/kor/sub02_01.do" title="다운로드"> 다운로드</a> 페이지로부터 KSB 툴박스(docker image 별도) 파일을 다운로드합니다.
  - ksb_toolbox_v1903.tar.gz : KSB 툴박스 파일
<br>

csle 사용자 계정의 home에 ksb_toolbox_v1903.tar.gz 압축을 해제합니다. 결과적으로 /home/csle/ksb-csle 폴더가 생성됩니다.

```sh
tar zxvf ksb_toolbox_v1903.tar.gz -C /home/csle/
```

/home/csle/ksb-csle 폴더는 다음의 하위 폴더들로 구성됩니다.
  - /bin : KSB 프레임워크 shell 스크립트 파일 저장 폴더
  - /components : tensorflow train 예제 프로그램 저장 폴더
  - /conf : KSB 프레임워크 환경설정 파일 저장 폴더  
  - /docker : KSB 툴박스 docker 컨테이너 실행 스크립트 저장 폴더
  - /examples : CLI(Command Line Interface)를 통해 워크플로우 생성 및 submit 할 수 있는 프로젝트 폴더
  - /jars : KSB 프레임워크 19.03 버전의 jar 파일 저장 폴더
  - /ksblib : python 전처리 라이브러리 폴더
  - /kubernetes : 쿠버네티스 환경설정 폴더
  - /logs : log 저장 폴더
  - /pyML : autoML python 프로젝트 폴더
  - /tools: 예제 테스트를 위한 프로그램(jmeter, hadoop, kafka, maven, .ssh, webToolkit_db)
<br>

Host PC에서 다음 명령으로 KSB 툴박스 docker image를 docker hub로부터 pull 합니다.

```sh
docker pull ksbframework/ksb_toolbox:19.03.3-miniconda3
```

Host PC에서 다음 명령으로 KSB 툴박스 docker image를 확인합니다.

**<span style="font-size: 10pt; color:red">[참고사항] 현재 2019.07.10일 docker image 버전은 19.03.3 버전이며, 추후 TAG 정보 및 SIZE 정보는 배포 버전에 따라 다를 수 있습니다**

```sh
csle@csle1:~/ksb-csle$ docker images
REPOSITORY                TAG                IMAGE ID      CREATED       SIZE
ksbframework/ksb_toolbox  19.03.3-miniconda3 f916162a0f16  7 days ago  7.86GB
```


## [Host PC] KSB 툴박스 docker 이미지내 SSH 키를 host pc에 복사하기
KSB 툴박스 docker 컨테이너의 인증키를 공유하기 위해 ksb-csle/tools 폴더에 있는 .ssh 폴더를 csle 홈에 카피합니다.

```sh
cp -r ~/ksb-csle/tools/.ssh/ /home/csle/
```
authorized_keys를 다른 계정이 읽고 쓸 수 없게 600 mode(소유자만 rw 가능)로 변경합니다.
```sh
chmod 700 ~/.ssh
chmod 600 ~/.ssh/authorized_keys
```
**<span style="font-size: 10pt; color:red">[참고사항] 만약 보안을 위해 새로운 SSH 키를 Host PC와 docker 컨테이너가 공유할 수 있습니다.** <a href="https://csleoss.etri.re.kr/images/contents/manual_1903/2019-07-18-ssh_key_new_generation.html" title="SSH키 생성 및 공유하기">[바로가기] SSH키 생성 및 공유하기</a>

## [Host PC] SSH 재시작하기
```sh
sudo service ssh restart
```

## [Host PC] ksb-csle 폴더 권한 변경하기
KSB 툴박스 docker 컨테이너에서 Host PC의 ksb-csle 폴더를 엑세스 할 수 있도록 폴더 권한을 변경합니다.
```sh
sudo chmod 777 -R /home/csle/ksb-csle/
```


## [HOST PC] startDockerCsle.sh 수정하기
바뀐 hostname을 이용하여 KSB 웹툴킷 docker 컨테이너를 실행하기 위해서 아래와 같이 수정합니다.
```sh
cd ~/ksb-csle/docker/19.03-host
vi startDockerCsle.sh
```

```sh
#!/bin/bash
sudo service postgresql stop

docker network rm csle_cluster csle_standalone
docker rm -f csle1

echo "csle1 slave container..."
echo "start csle1 slave container..."
docker run --rm -itd \
                --net=host \
                -v /home/csle/ksb-csle:/home/csle/ksb-csle \
                -v /etc/localtime:/etc/localtime:ro \
                -v /etc/timezone:/etc/timezone \
                --user=csle \
                --name=csle1 \
[수정]           --hostname=$HOSTNAME \
                ksbframework/ksb_toolbox:19.03.3-miniconda3 bash

docker exec -it csle1 bash
```

## [HOST PC] ksb 웹툴킷 실행하기
```sh
./startDockerCsle.sh
```

아래 부분은 docker 컨테이너 내부에서 환경설정을 수정하는 방법을 설명합니다.
먼저 docker 컨테이너의 ssh 포트를 변경합니다.

## [Docker 컨테이너] sshd_config port 변경하기
```sh
sudo vi /etc/ssh/sshd_config
```
```sh
Port 2243
# Authentication:
LoginGraceTime 120
PermitRootLogin yes
StrictModes yes
```

## [Docker 컨테이너] Hadoop 수정하기
바뀐 docker 컨테이너의 ssh 포트 (2243)을 이용해서 hadoop을 사용하기 위해 아래와 같이 환경설정 파일을 수정합니다.

```sh
cd
cd hadoop/etc/hadoop
```

### core-site.xml 수정하기
csle1을 csleserver로 수정합니다.
```sh
vi core-site.xml
:%s/csle1/csleserver/g
```

### master 수정하기
csle1을 csleserver로 수정합니다.
```sh
vi masters
:%s/csle1/csleserver/g
```

### slaves 수정하기
csle1을 csleserver로 수정합니다.
```sh
vi slaves
:%s/csle1/csleserver/g
```

### yarn-site.xml 수정하기
csle1을 csleserver로 수정합니다.
```sh
vi yarn-site.xml
:%s/csle1/csleserver/g
```

### hadoop-env.sh 추가하기
SSH 설정을 위해 아래의 한줄을 추가합니다.
```sh
vi hadoop-env.sh

export HADOOP_SSH_OPTS="-p 2243"
```
## [Docker 컨테이너] Hbase 수정하기
### hbase-site.xml 수정하기
csle1을 csleserver로 수정합니다.
```sh
cd ~/hbase/conf
vi hbase-site.xml
:%s/csle1/csleserver/g
```

### regionservers 수정하기
csle1을 csleserver로 수정합니다.
```sh
vi regionservers
:%s/csle1/csleserver/g
```
### hbase-env.sh 수정하기
SSH 설정을 위해 아래의 한줄을 추가합니다.
```sh
vi hbase-env.sh
export HBASE_SSH_OPTS="-p 2243"
```

## [Docker 컨테이너] startService.sh 수정하기
바뀐 ssh 2243 포트로 동작하기 위해 아래의 포트와 csle1 이름을 csleserver 이름으로 수정합니다.

```sh
cd
vi startService.sh
```
```sh
#!/bin/bash
export TERM=xterm
stty rows 36 cols 150
sudo service ssh restart
sudo service postgresql restart

bash /home/csle/zookeeper-3.4.9/bin/zkServer.sh start
[수정] ssh csle@master -p 2243 "cd /home/csle/zookeeper-3.4.9/bin; ./zkServer.sh start"
[수정] ssh csle@csleserver -p 2243 "cd /home/csle/zookeeper-3.4.9/bin; ./zkServer.sh start"
start-dfs.sh
start-yarn.sh
start-hbase.sh
/home/csle/kafka/bin/kafka-server-start.sh /home/csle/kafka/config/server.properties &
/home/csle/ui_of_csle/apache-tomcat-7.0.81/bin/catalina.sh start &
/home/csle/start-mongo.sh &
sleep 5
cd /home/csle/ksb-csle/bin
/home/csle/ksb-csle/bin/startKnowledge_service.sh localhost 9876
```

## [Docker 컨테이너] zookeeper 수정하기
csle1을 csleserver로 수정합니다.
```sh
vi ~/zookeeper/conf/zoo.cfg
```
```sh
server.1=csleserver:2888:3888
```

## [Docker 컨테이너] kafka 수정하기
csle1을 csleserver로 수정합니다.
```sh
vi ~/kafka/config/server.properties
```
```sh
advertised.listeners=PLAINTEXT://csleserver:9092
zookeeper.connect=csleserver:2181
```

### [Docker 컨테이너] HDFS 초기화하기
hostname이 변경되었으므로 hdfs namenode를 초기화해야 합니다.
[중 ]아래의 스크립트를 한줄씩 실행합니다.

```sh
cd
sudo rm -rf data
hdfs namenode -format
sudo service ssh start
start-dfs.sh
cd ksb-csle/bin/
./initHdfs.sh
stop-dfs.sh
sudo service ssh stop
```

### [Docker 컨테이너] ksb-csle/conf/ksb.conf 수정하기
```sh
vi ~/ksb-csle/conf/ksb.conf
```
```json
hadoop {
    home = "/hadoop/"
    master = "csleserver" [수정]
    masterIp = "192.168.1.102" [수정]
    port = "9000"
    hdfs {
        activated = "true"
        baseDir = "/user/"
        modelPath = "/model"
        datasetPath = "/dataset"
    }
    webhdfs {
        port = "50070"
        baseDir = "/webhdfs/v1"
    }
```

### [Docker 컨테이너] UI 웹툴킷 설정 수정하기
바뀐 hostname을 UI 웹툴킷에 적용하기 위해 아래와 같이 웹툴킷 UI에서 관련 정보를 수정합니다.

```sh
sudo service postgresql start
/home/csle/ui_of_csle/apache-tomcat-7.0.81/bin/catalina.sh start &
```

### [HOST PC] csleserver:8080 접속 및 로그인
크롬을 통해서 csleserver:8080 접속 및 로그인 합니다. 에러는 무시해도 됩니다.
로그인 후, 웹툴킷 메뉴에서 아래의 위치에 csle1을 csleserver로 변경합니다.

Management - system configuration- webhdfs.host : csleserver 수정

### [Docker 컨테이너] docker 이미지 저장하기 위해 tomcat 동작 stop하기
```sh
/home/csle/ui_of_csle/apache-tomcat-7.0.81/bin/catalina.sh stop
sudo service postgresql stop
```

### [HOST PC] docker 이미지 저장하기
위에서 지금까지 수정한 docker 컨테이너의 수정내용을 이미지로 저장합니다.
HOST PC의 터미널에서 실행하여야 합니다.

```sh
docker commit csle1 ksbframework/ksb_toolbox:19.03.3.1-miniconda3
```

### [HOST PC] startDockerCsle.sh 수정하기
바뀐 이미지를 이용하여 KSB 웹툴킷 docker 컨테이너를 실행하기 위해 아래와 같이 수정합니다.
```sh
#!/bin/bash
sudo service postgresql stop

docker network rm csle_cluster csle_standalone
docker rm -f csle1

echo "csle1 slave container..."

docker run --rm -itd \
                --net=host \
                -v /home/csle/ksb-csle:/home/csle/ksb-csle \
                -v /etc/localtime:/etc/localtime:ro \
                -v /etc/timezone:/etc/timezone \
                --user=csle \
                --name=csle1 \
                --hostname=$HOSTNAME \
[수정]           ksbframework/ksb_toolbox:19.03.3.1-miniconda3 bash

docker exec -it csle1 bash
```

### [HOST PC] ksb 툴박스 실행하기
```sh
cd ~/ksb-csle/docker/19.03-host$
./startDockerCsle.sh
```

### [Docker 컨테이너] KSB 툴박스 초기화하기

```sh
cd
./startService.sh
cd ~/ksb-csle/bin
./startKsbApiServing.sh
```

## [주의사항] 탑재되어 있는 예제를 실행하는 경우 수정해야 하는 사항
탑재되어 있는 예제에는 csle1 주소를 기준으로 작성되어 있습니다.
따라서, kafka reader / writer 예제에서 다음의 파라미터들의 변경이 필요합니다.
```sh
bootStrapServer 주소 : csleserver:9092
zooKeeperConnect 주소: csleserver:2181
```

아래 예제들에서 kafka reader/writer를 사용하므로 파라미터의 수정이 필요합니다.

```sh
2.5.7.RealtimeIngestToPredictInSingleEngine 	
2.5.8.RealtimeIngestToServingInTwoEngines
2.5.9.RealtimeIngestToServingWithKbInTwoEngines
2.5.12.TfStreamPredictionMnist
2.6.1.TrafficPreprocessing
2.6.6.TrafficStreamingPredict
2.6.7.TrafficEndToEnd
```

또한, hdfs://csle1:9000 주소에 file write 하는 예제들도 hdfs://csleserver:9000로 자신의 hostname에 맞도록 수정되어야 합니다.
아래 예제들의 파라미터의 수정이 필요합니다.

```sh
2.5.10.HourlyTensorflowTraining
2.5.14.TrafficPeriodicTrainAndK8sServingExample
2.6.2.TrafficTraining
2.6.4.TrafficStreamServing
```

[Part3] Random Forest example 예제를 실행하기 전, 아래와 같이 수정 후 실행합니다.
13번째 operator : RandomForestClassifierTrainOperator

randomForestTrainer -> traininfo 에서 csle1 --< csleserver로 변경
```sh
hdfs://csleserver:9000/user/ksbuser_etri_re_kr/model/occupancy
```

2.5.12.TfStreamPredictionMnist 예제의 경우, 테스트 코드를 수정한 후 테스트 합니다.
아래의 스크립트를 수정합니다.

```sh
vi ~/ksb-csle/examples/models/mnist/client/kafka-json/consume-mnist-output.sh
```

```sh
#!/bin/bash
KAFKA_CONSUMER=$KSB_HOME/tools/kafka_2.11-0.10.0.1/bin/kafka-console-consumer.sh
SERVER=csleserver:9092
TOPIC=mnist_output

$KAFKA_CONSUMER --zookeeper csleserver:2181 --bootstrap-server $SERVER --topic $TOPIC
```

아래의 스크립트를 수정합니다.
```sh
vi ~/ksb-csle/examples/models/mnist/client/kafka-json/publish-mnist-input.sh
```

```sh
#!/bin/bash

INPUT_FILE=mnist_input.json
KAFKA_PRODUCER=/home/csle/ksb-csle/tools/kafka_2.11-0.10.0.1/bin/kafka-console-producer.sh
[수정] BROKERS=csleserver:9092
TOPIC=mnist_input

cat $INPUT_FILE | $KAFKA_PRODUCER --broker-list $BROKERS --topic $TOPIC
```

2.5.13.ConvergedServingEndToEndExample 예제를 실행하기 전, 아래와 같이 수정 후 실행합니다.

```sh
vi ~/ksb-csle/ksblib/ksblib/dockerize/base.py
:%s/csle1/csleserver/g
:%s/2243/22/g
```
